@@ -36,9 +36,10 @@
 #include "NetworkManager.h"
 #include "nm-utils.h"
 #include "NetworkManagerUtils.h"
-#include "NetworkManagerDevice.h"
+#include "nm-device.h"
+#include "nm-device-802-3-ethernet.h"
+#include "nm-device-802-11-wireless.h"
 #include "NetworkManagerPolicy.h"
-#include "NetworkManagerWireless.h"
 #include "NetworkManagerDbus.h"
 #include "NetworkManagerAP.h"
 #include "NetworkManagerAPList.h"
@@ -48,16 +49,18 @@
 #include "nm-dbus-vpn.h"
 #include "nm-netlink-monitor.h"
 #include "nm-dhcp-manager.h"
+#include "nm-logging.h"
 
 #define NM_WIRELESS_LINK_STATE_POLL_INTERVAL (5 * 1000)
 
+#define NM_DEFAULT_PID_FILE	LOCALSTATEDIR"/run/NetworkManager.pid"
+
 /*
  * Globals
  */
 static NMData		*nm_data = NULL;
 
 static gboolean sigterm_pipe_handler (GIOChannel *src, GIOCondition condition, gpointer data);
-static void sigterm_handler (int signum);
 static void nm_data_free (NMData *data);
 
 /*
@@ -129,7 +132,7 @@ NMDevice * nm_create_device_and_add_to_list (NMData *data, const char *udi, cons
 		if (nm_try_acquire_mutex (data->dev_list_mutex, __FUNCTION__))
 		{
 			nm_info ("Now managing %s device '%s'.",
-				nm_device_is_wireless (dev) ? "wireless" : "wired", nm_device_get_iface (dev));
+				nm_device_is_802_11_wireless (dev) ? "wireless (802.11)" : "wired Ethernet (802.3)", nm_device_get_iface (dev));
 
 			data->dev_list = g_slist_append (data->dev_list, dev);
 			nm_device_deactivate (dev);
@@ -143,10 +146,10 @@ NMDevice * nm_create_device_and_add_to_list (NMData *data, const char *udi, cons
 		{
 			/* If we couldn't add the device to our list, free its data. */
 			nm_warning ("could not acquire device list mutex." );
-			nm_device_unref (dev);
+			g_object_unref (G_OBJECT (dev));
 			dev = NULL;
 		}
-	} else nm_warning ("could not allocate device data." );
+	}
 
 	return (dev);
 }
@@ -164,11 +167,10 @@ void nm_remove_device (NMData *data, NMDevice *dev)
 	g_return_if_fail (dev != NULL);
 
 	nm_device_set_removed (dev, TRUE);
-	nm_device_deactivate (dev);
-	nm_device_worker_thread_stop (dev);
+	nm_device_stop (dev);
 	nm_dbus_schedule_device_status_change_signal (data, dev, NULL, DEVICE_REMOVED);
 
-	nm_device_unref (dev);
+	g_object_unref (G_OBJECT (dev));
 
 	/* Remove the device entry from the device list and free its data */
 	data->dev_list = g_slist_remove (data->dev_list, dev);
@@ -235,7 +237,7 @@ static void nm_hal_device_added (LibHalContext *ctx, const char *udi)
 	 */
 	if ((iface = nm_get_device_interface_from_hal (data->hal_ctx, udi)))
 	{
-		nm_create_device_and_add_to_list (data, udi, iface, FALSE, DEVICE_TYPE_DONT_KNOW);
+		nm_create_device_and_add_to_list (data, udi, iface, FALSE, DEVICE_TYPE_UNKNOWN);
 		g_free (iface);
 	}
 }
@@ -285,7 +287,7 @@ static void nm_hal_device_new_capability (LibHalContext *ctx, const char *udi, c
 
 		if ((iface = nm_get_device_interface_from_hal (data->hal_ctx, udi)))
 		{
-			nm_create_device_and_add_to_list (data, udi, iface, FALSE, DEVICE_TYPE_DONT_KNOW);
+			nm_create_device_and_add_to_list (data, udi, iface, FALSE, DEVICE_TYPE_UNKNOWN);
 			g_free (iface);
 		}
 	}
@@ -324,7 +326,7 @@ void nm_add_initial_devices (NMData *data)
 
 			if ((iface = nm_get_device_interface_from_hal (data->hal_ctx, net_devices[i])))
 			{
-				nm_create_device_and_add_to_list (data, net_devices[i], iface, FALSE, DEVICE_TYPE_DONT_KNOW);
+				nm_create_device_and_add_to_list (data, net_devices[i], iface, FALSE, DEVICE_TYPE_UNKNOWN);
 				g_free (iface);
 			}
 		}
@@ -376,8 +378,6 @@ void nm_schedule_state_change_signal_broadcast (NMData *data)
  */
 static NMData *nm_data_new (gboolean enable_test_devices)
 {
-	struct sigaction	action;
-	sigset_t			block_mask;
 	NMData *			data;
 	GSource *			iosource;
 	
@@ -386,25 +386,21 @@ static NMData *nm_data_new (gboolean enable_test_devices)
 	data->main_context = g_main_context_new ();
 	data->main_loop = g_main_loop_new (data->main_context, FALSE);
 
-	if (pipe(data->sigterm_pipe) < 0)
+	/* Allow clean shutdowns by having the thread which receives the signal
+	 * notify the main thread to quit, rather than having the receiving
+	 * thread try to quit the glib main loop.
+	 */
+	if (pipe (data->sigterm_pipe) < 0)
 	{
 		nm_error ("Couldn't create pipe: %s", g_strerror (errno));
-		exit (EXIT_FAILURE);
+		return NULL;
 	}
-
 	data->sigterm_iochannel = g_io_channel_unix_new (data->sigterm_pipe[0]);
 	iosource = g_io_create_watch (data->sigterm_iochannel, G_IO_IN | G_IO_ERR);
 	g_source_set_callback (iosource, (GSourceFunc) sigterm_pipe_handler, data, NULL);
 	g_source_attach (iosource, data->main_context);
 	g_source_unref (iosource);
 
-	action.sa_handler = sigterm_handler;
-	sigemptyset (&block_mask);
-	action.sa_mask = block_mask;
-	action.sa_flags = 0;
-	sigaction (SIGINT, &action, NULL);
-	sigaction (SIGTERM, &action, NULL);
-
 	/* Initialize the device list mutex to protect additions/deletions to it. */
 	data->dev_list_mutex = g_mutex_new ();
 	data->dialup_list_mutex = g_mutex_new ();
@@ -442,7 +438,7 @@ static void device_stop_and_free (NMDevice *dev, gpointer user_data)
 
 	nm_device_set_removed (dev, TRUE);
 	nm_device_deactivate (dev);
-	nm_device_unref (dev);
+	g_object_unref (G_OBJECT (dev));
 }
 
 
@@ -487,16 +483,9 @@ static void nm_data_free (NMData *data)
 	memset (data, 0, sizeof (NMData));
 }
 
-static void sigterm_handler (int signum)
+int nm_get_sigterm_pipe (void)
 {
-        int ignore;
-
-	/* FIXME: This line is probably not a great 
-	 * thing to have in a signal handler
-	 */
-	nm_info ("Caught SIGINT/SIGTERM");
-
-	ignore = write (nm_data->sigterm_pipe[1], "X", 1);
+	return nm_data->sigterm_pipe[1];
 }
 
 static gboolean sigterm_pipe_handler (GIOChannel *src, GIOCondition condition, gpointer user_data)
@@ -508,109 +497,26 @@ static gboolean sigterm_pipe_handler (GIOChannel *src, GIOCondition condition, g
 	return FALSE;
 }
 
-/*
- * nm_print_usage
- *
- * Prints program usage.
- *
- */
-static void nm_print_usage (void)
-{
-	fprintf (stderr, "\n" "usage : NetworkManager [--no-daemon] [--help]\n");
-	fprintf (stderr,
-		"\n"
-		"        --no-daemon             Don't become a daemon\n"
-		"        --enable-test-devices   Allow dummy devices to be created via DBUS methods [DEBUG]\n"
-		"        --help                  Show this information and exit\n"
-		"\n"
-		"NetworkManager monitors all network connections and automatically\n"
-		"chooses the best connection to use.  It also allows the user to\n"
-		"specify wireless access points which wireless cards in the computer\n"
-		"should associate with.\n"
-		"\n");
-}
-
-/*
- * nm_poll_and_update_wireless_link_state
- *
- * Called every 2s to poll wireless cards and determine if they have a link
- * or not.
- *
- */
-static gboolean nm_poll_and_update_wireless_link_state (NMData *data)
-{
-	GSList *		elt;
-	GSList *		copy = NULL;
-	NMDevice *	dev;
-
-	g_return_val_if_fail (data != NULL, TRUE);
-
-	if ((data->wireless_enabled == FALSE) || (data->asleep == TRUE))
-		return (TRUE);
-
-	/* Copy device list and ref devices to keep them around */
-	if (nm_try_acquire_mutex (data->dev_list_mutex, __FUNCTION__))
-	{
-		for (elt = data->dev_list; elt; elt = g_slist_next (elt))
-		{
-			if ((dev = (NMDevice *)(elt->data)))
-			{
-				nm_device_ref (dev);
-				copy = g_slist_append (copy, dev);
-			}
-		}
-		nm_unlock_mutex (data->dev_list_mutex, __FUNCTION__);
-	}
-
-	for (elt = copy; elt; elt = g_slist_next (elt))
-	{
-		if ((dev = (NMDevice *)(elt->data)))
-		{
-			if (nm_device_is_wireless (dev) && !nm_device_is_activating (dev))
-			{
-				nm_device_set_link_active (dev, nm_device_probe_link_state (dev));
-				nm_device_update_signal_strength (dev);
-			}
-			nm_device_unref (dev);
-		}
-	}
-
-	g_slist_free (copy);
-	
-	return TRUE;
-}
-
-static void nm_monitor_wireless_link_state (NMData *data)
-{
-	GSource *link_source;
-	link_source = g_timeout_source_new (NM_WIRELESS_LINK_STATE_POLL_INTERVAL);
-	g_source_set_callback (link_source, 
-			       (GSourceFunc) nm_poll_and_update_wireless_link_state, 
-			       data, NULL);
-	g_source_attach (link_source, nm_data->main_context);
-	g_source_unref (link_source);
-}
-
 static void nm_device_link_activated (NmNetlinkMonitor *monitor, const gchar *interface_name, NMData *data)
 {
 	NMDevice *dev = NULL;
 
-	if (nm_try_acquire_mutex (data->dev_list_mutex, __FUNCTION__))
+	if (nm_try_acquire_mutex (data->dev_list_mutex, __func__))
 	{
 		if ((dev = nm_get_device_by_iface (data, interface_name)))
-			nm_device_ref (dev);
-		nm_unlock_mutex (data->dev_list_mutex, __FUNCTION__);
+			g_object_ref (G_OBJECT (dev));
+		nm_unlock_mutex (data->dev_list_mutex, __func__);
 	}
 
 	/* Don't do anything if we already have a link */
 	if (dev)
 	{
-		if (nm_device_is_wired (dev) && !nm_device_has_active_link (dev))
+		if (nm_device_is_802_3_ethernet (dev) && !nm_device_has_active_link (dev))
 		{
-			nm_device_set_link_active (dev, TRUE);
+			nm_device_set_active_link (dev, TRUE);
 			nm_policy_schedule_device_change_check (data);
 		}
-		nm_device_unref (dev);
+		g_object_unref (G_OBJECT (dev));
 	}
 }
 
@@ -618,18 +524,18 @@ static void nm_device_link_deactivated (NmNetlinkMonitor *monitor, const gchar *
 {
 	NMDevice *dev = NULL;
 
-	if (nm_try_acquire_mutex (data->dev_list_mutex, __FUNCTION__))
+	if (nm_try_acquire_mutex (data->dev_list_mutex, __func__))
 	{
 		if ((dev = nm_get_device_by_iface (data, interface_name)))
-			nm_device_ref (dev);
-		nm_unlock_mutex (data->dev_list_mutex, __FUNCTION__);
+			g_object_ref (G_OBJECT (dev));
+		nm_unlock_mutex (data->dev_list_mutex, __func__);
 	}
 
 	if (dev)
 	{
-		if (nm_device_is_wired (dev))
-			nm_device_set_link_active (dev, FALSE);
-		nm_device_unref (dev);
+		if (nm_device_is_802_3_ethernet (dev))
+			nm_device_set_active_link (dev, FALSE);
+		g_object_unref (G_OBJECT (dev));
 	}
 }
 
@@ -683,58 +589,6 @@ nm_monitor_wired_link_state (NMData *data)
 	data->netlink_monitor = monitor;
 }
 
-static void
-nm_info_handler (const gchar	*log_domain,
-		GLogLevelFlags	 log_level,
-		const gchar	*message,
-		gboolean 	 is_daemon)
-{
-	int syslog_priority;	
-
-	switch (log_level)
-	{
-		case G_LOG_LEVEL_ERROR:
-			syslog_priority = LOG_CRIT;
-		break;
-
-		case G_LOG_LEVEL_CRITICAL:
-			syslog_priority = LOG_ERR;
-		break;
-
-		case G_LOG_LEVEL_WARNING:
-			syslog_priority = LOG_WARNING;
-		break;
-
-		case G_LOG_LEVEL_MESSAGE:
-			syslog_priority = LOG_NOTICE;
-
-		case G_LOG_LEVEL_DEBUG:
-			syslog_priority = LOG_DEBUG;
-		break;
-
-		case G_LOG_LEVEL_INFO:
-		default:
-			syslog_priority = LOG_INFO;
-		break;
-	}
-
-	syslog (syslog_priority, "%s", message);
-}
-
-static void
-nm_set_up_log_handlers (gboolean become_daemon)
-{
-	if (become_daemon)
-		openlog (G_LOG_DOMAIN, LOG_CONS, LOG_DAEMON);
-	else
-		openlog (G_LOG_DOMAIN, LOG_CONS | LOG_PERROR, LOG_USER);
-
-	g_log_set_handler (G_LOG_DOMAIN, 
-	                   G_LOG_LEVEL_MASK,
-			   (GLogFunc) nm_info_handler,
-			   GINT_TO_POINTER (become_daemon));
-}
-
 
 static LibHalContext *nm_get_hal_ctx (NMData *data)
 {
@@ -811,6 +665,50 @@ void nm_hal_deinit (NMData *data)
 	}
 }
 
+
+static void
+write_pidfile (const char *pidfile)
+{
+ 	char pid[16];
+	int fd;
+ 
+	if ((fd = open (pidfile, O_CREAT|O_WRONLY|O_TRUNC, 00644)) < 0)
+	{
+		nm_warning ("Opening %s failed: %s", pidfile, strerror (errno));
+		return;
+	}
+ 	snprintf (pid, sizeof (pid), "%d", getpid ());
+	if (write (fd, pid, strlen (pid)) < 0)
+		nm_warning ("Writing to %s failed: %s", pidfile, strerror (errno));
+	if (close (fd))
+		nm_warning ("Closing %s failed: %s", pidfile, strerror (errno));
+}
+
+
+/*
+ * nm_print_usage
+ *
+ * Prints program usage.
+ *
+ */
+static void nm_print_usage (void)
+{
+	fprintf (stderr, "\n" "usage : NetworkManager [--no-daemon] [--pid-file=<file>] [--help]\n");
+	fprintf (stderr,
+		"\n"
+		"        --no-daemon             Don't become a daemon\n"
+		"        --pid-file=<path>       Specify the location of a PID file\n"
+		"        --enable-test-devices   Allow dummy devices to be created via DBUS methods [DEBUG]\n"
+		"        --help                  Show this information and exit\n"
+		"\n"
+		"NetworkManager monitors all network connections and automatically\n"
+		"chooses the best connection to use.  It also allows the user to\n"
+		"specify wireless access points which wireless cards in the computer\n"
+		"should associate with.\n"
+		"\n");
+}
+
+
 /*
  * main
  *
@@ -820,6 +718,8 @@ int main( int argc, char *argv[] )
 	gboolean		become_daemon = TRUE;
 	gboolean		enable_test_devices = FALSE;
 	char *		owner;
+	char *		pidfile = NULL;
+	char *		user_pidfile = NULL;
 	
 	if (getuid () != 0)
 	{
@@ -837,6 +737,7 @@ int main( int argc, char *argv[] )
 		static struct option options[] = {
 			{"no-daemon",			0, NULL, 0},
 			{"enable-test-devices",	0, NULL, 0},
+			{"pid-file",			1, NULL, 0},
 			{"help",				0, NULL, 0},
 			{NULL,				0, NULL, 0}
 		};
@@ -858,6 +759,8 @@ int main( int argc, char *argv[] )
 					become_daemon = FALSE;
 				else if (strcmp (opt, "enable-test-devices") == 0)
 					enable_test_devices = TRUE;
+				else if (strcmp (opt, "pid-file") == 0)
+					user_pidfile = g_strdup (optarg);
 				break;
 
 			default:
@@ -867,22 +770,28 @@ int main( int argc, char *argv[] )
 		}
 	}
 
-	if (become_daemon && daemon (0, 0) < 0)
+	if (become_daemon)
 	{
-		int saved_errno;
+		if (daemon (0, 0) < 0)
+		{
+			int saved_errno;
 
-		saved_errno = errno;
-		nm_error ("NetworkManager could not daemonize: %s [error %u]",
-			  g_strerror (saved_errno), saved_errno);
-		exit (EXIT_FAILURE);
+			saved_errno = errno;
+			nm_error ("NetworkManager could not daemonize: %s [error %u]",
+				  g_strerror (saved_errno), saved_errno);
+			exit (EXIT_FAILURE);
+		}
+
+		pidfile = user_pidfile ? user_pidfile : NM_DEFAULT_PID_FILE;
+		write_pidfile (pidfile);
 	}
 
 	g_type_init ();
 	if (!g_thread_supported ())
 		g_thread_init (NULL);
 	dbus_g_thread_init ();
-
-	nm_set_up_log_handlers (become_daemon);
+	
+	nm_logging_setup (become_daemon);
 	nm_info ("starting...");
 
 	nm_system_init();
@@ -903,7 +812,6 @@ int main( int argc, char *argv[] )
 			  "Either dbus is not running, or the "
 			  "NetworkManager dbus security policy "
 			  "was not loaded.");
-		nm_data_free (nm_data);
 		exit (EXIT_FAILURE);
 	}
 
@@ -939,7 +847,6 @@ int main( int argc, char *argv[] )
 	nm_system_enable_loopback ();
 
 	/* Create watch functions that monitor cards for link status. */
-	nm_monitor_wireless_link_state (nm_data);
 	nm_monitor_wired_link_state (nm_data);
 
 	/* Get modems, ISDN, and so on's configuration from the system */
@@ -952,6 +859,12 @@ int main( int argc, char *argv[] )
 
 	nm_print_open_socks ();
 	nm_data_free (nm_data);
+	nm_logging_shutdown ();
+
+	/* Clean up pidfile */
+	if (pidfile)
+		unlink (pidfile);
+	g_free (user_pidfile);
 
 	exit (0);
 }
