@@ -24,8 +24,10 @@
 
 #include <glib.h>
 #include "NetworkManager.h"
-#include "NetworkManagerDevice.h"
-
+#include "NetworkManagerMain.h"
+#include "NetworkManagerAP.h"
+#include "nm-device.h"
+#include "nm-device-802-11-wireless.h"
 
 typedef struct NMAccessPointList	NMAccessPointList;
 typedef struct NMAPListIter		NMAPListIter;
@@ -49,8 +51,7 @@ void				nm_ap_list_copy_properties		(NMAccessPointList *dest, NMAccessPointList
 void				nm_ap_list_copy_essids_by_address	(NMAccessPointList *dest, NMAccessPointList *source);
 void				nm_ap_list_copy_one_essid_by_address	(NMAccessPoint *ap, NMAccessPointList *search_list);
 
-void				nm_ap_list_diff				(NMData *data, NMDevice *dev, NMAccessPointList *old, NMAccessPointList *new);
-gboolean			nm_ap_list_merge_scanned_ap		(NMAccessPointList *list, NMAccessPoint *merge_ap, gboolean *new, gboolean *strength_changed);
+gboolean			nm_ap_list_merge_scanned_ap		(NMDevice80211Wireless *dev, NMAccessPointList *list, NMAccessPoint *merge_ap);
 
 NMNetworkType		nm_ap_list_get_type				(NMAccessPointList *list);
 
